//--------------------------------------------------------------------  //000
//                                                                      //001
// DISCOUNT: Display-Controller                                         //002
//                                                                      //003
// Stellt einen 8-Bit-Speicher auf einem Video-Monitor monochrom dar    //004
// (Aufloesung 640x192, Bildfrequenz 50 Hz).                            //005
//                                                                      //006
// Das Modell ist in den Abschnitten 13.1-3 ausfuehrlich dokumentiert.  //007
//                                                                      //008
//--------------------------------------------------------------------  //009
                                                                        //010
//--------------------------------------------------------------------  //031
//                                                                      //032
// Modul discount                                                       //033
//                                                                      //034
// Display-Controller (Cathode-Ray-Tube-Controller)                     //035
//                                                                      //036
//--------------------------------------------------------------------  //037
                                                                        //038
`include "discount_defs.v"

module discount (ADDR, PIXEL, CSYNC, HSYNC, VSYNC, DATA, CLOCK, NRST);  //039
                                                                        //040
  // Ausgaenge                                                          //041
  output [`Asz-1:0] ADDR;   // Adresse fuer Bildspeicher                //042
  output            PIXEL,  // Bildpunkt (0: dunkel, 1: hell)           //043
                    CSYNC,  // kombinierte Synchronisation              //044
                            //   (HSYNC und VSYNC)                      //045
                    HSYNC,  // horizontale (Zeilen-)Synchronisation     //046
                            //   (1-aktiv)                              //047
                    VSYNC;  // vertikale (Bild-)Synchronisation         //048
                            //   (1-aktiv)                              //049
                                                                        //050
  // Eingaenge                                                          //051
  input  [`Dsz-1:0] DATA;   // Daten aus dem Bildspeicher               //052
  input             CLOCK,  // Takt (einphasig)                         //053
                    NRST;   // Initialisierung (0-aktiv)                //054
                                                                        //055
  // Port-Typen                                                         //056
  wire   [`Asz-1:0] ADDR;                                               //057
  wire   [`Dsz-1:0] DATA;                                               //058
  wire              PIXEL,                                              //059
                    CSYNC,                                              //060
                    HSYNC,                                              //061
                    VSYNC,                                              //062
                    CLOCK,                                              //063
                    NRST;                                               //064
                                                                        //065
  // interne Variablen                                                  //066
  wire              ADRIE,  // Freigabe: Bilddaten uebernehmen und      //067
                            //   Bildadresse erhoehen                   //068
                            //   (address increment enable)             //069
                    NXTAD,  // Bildadresse hochzaehlen                  //070
                    NXTLN;  // Zeile hochzaehlen                        //071
                                                                        //072
  //                                                                    //073
  // kombinierte Synchronisation                                        //074
  //                                                                    //075
  assign CSYNC = HSYNC ^ VSYNC;                                         //076
                                                                        //077
  //                                                                    //078
  // Instanzen                                                          //079
  //                                                                    //080
  hcount HCOUNT (NXTAD, NXTLN, HSYNC, CLOCK, NRST);                     //081
  vcount VCOUNT (ADRIE, VSYNC, NXTLN, CLOCK, NRST);                     //082
  memacc MEMACC (ADDR,  PIXEL, DATA,  ADRIE, NXTAD, CLOCK, NRST);       //083
                                                                        //084
endmodule  // discount                                                  //085
