//--------------------------------------------------------------------  //308
//                                                                      //309
// Modul memacc                                                         //310
//                                                                      //311
// Adressen fuer den Bildspeicher generieren, Bilddaten als Bytes       //312
// lesen und als Punkte ausgeben                                        //313
//                                                                      //314
//--------------------------------------------------------------------  //315
//                                                                      //316
// Schnittstelle                                                        //317
//                                                                      //318
//  ADDR   Adresse fuer Bildspeicher                                    //319
//  PIXEL  Bildpunkt (0: dunkel, 1: hell)                               //320
//  DATA   Daten aus dem Bildspeicher                                   //321
//  ADRIE  Freigabe: Bilddaten uebernehmen und Bildadresse erhoehen     //322
//         (address increment enable)                                   //323
//  NXTAD  Daten uebernehmen, Bildadresse hochzaehlen                   //324
//  CLOCK  Takt                                                         //325
//  NRST   Initialisierung (0-aktiv)                                    //326
//                                                                      //327
//--------------------------------------------------------------------  //328
//                                                                      //329
// Timing und Funktion:                                                 //330
//                                                                      //331
// Bei einer positiven Taktflanke und ADRIE=NXTAD=1 werden das          //332
// Bildpunkt-Schieberegister mit den momentanen Bildspeicherdaten       //333
// geladen und die Bildspeicheradresse erhoeht. Letztere liegt im       //334
// Bereich von 0 bis `Msz, worauf wieder 0 folgt.                       //335
// Ist ADRIE oder NXTAD 0, wird das Bildpunkt-Schieberegister           //336
// weitergeschoben und beim niedrigstwertigen Bit mit 0 gefuellt.       //337
//                                                                      //338
//--------------------------------------------------------------------  //339
                                                                        //340
`include "discount_defs.v"

module memacc (ADDR, PIXEL, DATA, ADRIE, NXTAD, CLOCK, NRST);           //341
                                                                        //342
  // Ausgaenge                                                          //343
  output [`Asz-1:0] ADDR;   // Adresse fuer Bildspeicher                //344
  output            PIXEL;  // Bildpunkt                                //345
                                                                        //346
  // Eingaenge                                                          //347
  input  [`Dsz-1:0] DATA;   // Daten aus dem Bildspeicher               //348
  input             ADRIE,  // Freigabe: Bildadresse erhoehen           //349
                    NXTAD,  // Bildadresse hochzaehlen                  //350
                    CLOCK,  // Takt                                     //351
                    NRST;   // Initialisierung (0-aktiv)                //352
                                                                        //353
  // Port-Typen                                                         //354
  reg    [`Asz-1:0] ADDR;                                               //355
  wire   [`Dsz-1:0] DATA;                                               //356
  wire              PIXEL,                                              //357
                    ADRIE,                                              //358
                    NXTAD,                                              //359
                    CLOCK,                                              //360
                    NRST;                                               //361
                                                                        //362
  // interne Variablen                                                  //363
  reg    [`Dsz-1:0] PIXSR;  // Bildpunkt-Schieberegister                //364
                                                                        //365
  //                                                                    //366
  // Bildpunkt aus Schieberegister abgreifen                            //367
  //                                                                    //368
  assign PIXEL = PIXSR[`Dsz-1];                                         //369
                                                                        //370
  //                                                                    //371
  // Bildspeicheradresse aktualisieren                                  //372
  //                                                                    //373
  always @(posedge CLOCK or negedge NRST) begin                         //374
    if (NRST == 0)                    ADDR <= 0;  // Reset              //375
    else                                                                //376
      if ((NXTAD == 1) && (ADRIE == 1)) begin     // neue Adresse       //377
        if (ADDR == `Msz-1) ADDR <= 0;            // Ende Bildspeicher  //378
        else                ADDR <= ADDR+1;                             //379
      end                                                               //380
  end                                                                   //381
                                                                        //382
  //                                                                    //383
  // Bildpunkt-Schieberegister aktualisieren                            //384
  //                                                                    //385
  always @(posedge CLOCK or negedge NRST) begin                         //386
    if (NRST == 0) PIXSR <= 0;                    // Reset              //387
    else                                                                //388
      if ((NXTAD == 1) && (ADRIE == 1))                                 //389
        PIXSR <= DATA;                            // neues Byte laden   //390
      else PIXSR <= {PIXSR[`Dsz-2:0], 1'b0};      // schieben           //391
  end                                                                   //392
                                                                        //393
endmodule // memacc                                                     //394
